/*-------------------------------------------------------------------------------
Titre : ESP8266 : LittleFS + jQuery
// le : 13/5/2021
// IDE : VSCode + PlatformIO
// Fichier : main.cpp
// Contrôle de la température dans une serre et mesure de l'humidité
---------------------------------------------------------------------------------*/
// Placé par PlatformIO
#include <Arduino.h>
// Système de fichiers
#include <LittleFS.h>
// Connexion au wifi
#include <ESP8266WiFi.h>
// mDNS pour la résolution des noms des hôtes
#include <ESP8266mDNS.h>
//  EEPROM : émule une EEPROM dans l'ESP8266
#include <EEPROM.h>
// Serveur HTTP
#include <ESP8266WebServer.h>
// Capteur BME280 SparkFun : température, humidité, pression
#include <SparkFunBME280.h>

#define Led LED_BUILTIN
#define RelaisChauffage 16 // D3 de la carte Particle

// Variables globales
int Temperature = 0;
int Humidite = 0;
// ---------------------------------------------------------------------------------
// Structure pour la configuration de la connexion au réseau wifi
struct EEconf
{                      // Les champs sont remplis avec les données préalablement placées en EEPROM émulée
                       // en utilisant le croquis infoClientMQTT_ESP8266.ino
  char ssid[32];       // SSID du réseau. Exemple : SynBoxLAN,
  char password[64];   // Mot de passe du réseau. Exemple : 12345678
  char myhostname[32]; // Nom donné au client MQTT. Exemple : ESP8266_1
} readconf;

// Objet pour la connexion au réseau wifi
WiFiClient espClient;
// Création d'un serveur HTTP
ESP8266WebServer server(80);
// Objet capteur BME280
BME280 sparkfunBME280;

// =================================================================================
// Fonction de connexion au Wifi
// ---------------------------------------------------------------------------------
void setup_wifi()
{
  // Mode station
  WiFi.mode(WIFI_STA);
  Serial.println();
  Serial.print("Tentative de connexion à ");
  Serial.println(readconf.ssid);
  // Connexion au Wifi
  WiFi.begin(readconf.ssid, readconf.password);
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(5000);
    Serial.print(".");
  }
  // Affichage
  Serial.println("");
  Serial.println("Connexion au Wifi ok");
  Serial.print("MAC: ");
  Serial.println(WiFi.macAddress());
  Serial.print("Adresse IP : ");
  Serial.println(WiFi.localIP());
  // Configuration de mDNS
  WiFi.hostname(readconf.myhostname);
  if (!MDNS.begin(readconf.myhostname))
  {
    Serial.println("Erreur de configuration mDNS !");
  }
  else
  {
    Serial.println("Répondeur mDNS démarré");
    Serial.println(readconf.myhostname);
  }
}

// =================================================================================
// Fonctions déclenchées en réponse aux requêtes HTTP
// ---------------------------------------------------------------------------------
void handle404Error()
{
  String message = "ERREUR 404 \n Fichier inconnu de LittleFS \n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += " \n";
  server.send(404, "text/plain,message", message);
  Serial.println(message);
}
// Gestion des I/O
// ---------------------------------------------------------------------------------
void handleHeater()
{
  String Commande = server.arg("chauffage"); // On recupére la valeur de l’argument
  Serial.println(Commande);
  if (Commande == "on")
  {
    digitalWrite(Led, LOW);
    digitalWrite(RelaisChauffage, HIGH);
  }
  else
  {
    digitalWrite(Led, HIGH);
    digitalWrite(RelaisChauffage, LOW);
  }
  server.send(200, "text/plane", Commande);
}

void handleMeasures()
{
  char msgTemp[16], msgHum[16];
  float temperature = sparkfunBME280.readTempC();
  dtostrf(temperature, 5, 1, msgTemp);
  float humidite = sparkfunBME280.readFloatHumidity();
  dtostrf(humidite, 5, 1, msgHum);
  String msginfos = "[{\"temp\":";
  msginfos += msgTemp;
  msginfos += ",";
  msginfos += "\"hum\":";
  msginfos += msgHum;
  msginfos += "}]";
  Serial.println(msginfos);
  server.send(200, "text/plane", msginfos);
}
// =================================================================================
void setup()
{
  // Configuation des I/O
  pinMode(Led, OUTPUT);
  digitalWrite(Led, HIGH); // Actif à l'état bas
  pinMode(RelaisChauffage, OUTPUT);
  digitalWrite(RelaisChauffage, LOW); // Actif à l'état haut

  // Configuration du moniteur série
  Serial.begin(115200);
  delay(500);

  // Lecture des paramètres sauvegardés par ARD_ESP_SauveInfosClientMqtt.ino
  EEPROM.begin(sizeof(readconf));
  EEPROM.get(0, readconf);

  // Connexion au Wifi
  setup_wifi();

  // Initialisation du bus I2C (la bibliothèque BME280 sparkfun ne le fait pas !)
  Wire.begin();
  // Test de la communication avec le capteur BME280
  if (sparkfunBME280.beginI2C() == false) // Début de la communication sur l'I2C
  {
    Serial.println("Le capteur ne répond pas");
    while (1)
      ; //TODO: A remplacer par chargement de la page et test des capteurs
  }

  // Démarrage du système de fichiers
  if (!LittleFS.begin())
  {
    Serial.println("Erreur initialisation LittleFS");
  }

  // Gestion des requêtes et démarrage du serveur HTTP
  server.serveStatic("/", LittleFS, "/index.html");
  server.serveStatic("/images/ajax-loader.gif",LittleFS,"/images/ajax-loader.gif");
  server.serveStatic("/index.html", LittleFS, "/index.html");
  server.serveStatic("/jquery.mobile-1.2.0.min.css", LittleFS, "/jquery.mobile-1.2.0.min.css");
  server.serveStatic("/jquery-1.8.2.min.js", LittleFS, "/jquery-1.8.2.min.js");
  server.serveStatic("/jquery.mobile-1.2.0.min.js", LittleFS, "/jquery.mobile-1.2.0.min.js");
  server.serveStatic("/plante.png", LittleFS, "/plante.png");
  server.on("/setHeater", handleHeater);     // gère la LED_BUILTIN de la carte
  server.on("/getMeasures", handleMeasures); // gère la mesure et l'envoi des grandeurs physiques
  server.onNotFound(handle404Error);         // page non trouvée
  server.begin();
}

void loop()
{
  // Gestion serveur HTTP
  server.handleClient();
}